/* Copyright (C) Kevin Vance, June 26 2003.
 * Email winnocence@peacefire.org
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <windows.h>

#include "about.h"
#include "resource.h"
#include "hostsfile.h"

const char title_version[] = "WINnocence 2.0";
const char winnocence_class[] = "winnocenceClass";
const char instructions[] = "Enter the list of sites you wish to block, "\
			    "one per line.";
char *blocked_sites;

/* Save menuitem was activated. */
int save(HWND hwnd)
{
	/* Allocate buffer for input, and load it. */
	HWND edit = GetDlgItem(hwnd, IDC_MAIN_EDIT);
	int length = GetWindowTextLength(edit) + 3;
	blocked_sites = realloc(blocked_sites, length);
	GetDlgItemText(hwnd, IDC_MAIN_EDIT, blocked_sites, length);
	/* Check if the text does not end with a newline. */
	if(blocked_sites[length-5] != '\r' ||
	   blocked_sites[length-4] != '\n')
		strcat(blocked_sites, "\r\n");
	/* Try to save to hosts file. */
	if(put_winnocence_lines(blocked_sites)) {
		SendMessage(edit, EM_SETMODIFY, FALSE, 0);
		MessageBox(NULL, "WINnocence has updated successfully.",
				"Notice", MB_ICONINFORMATION | MB_OK);
		return 1;
	} else {
		MessageBox(NULL, "WINnocence could not save your changes!",
				"Error!", MB_ICONERROR | MB_OK);
		return 0;
	}
}

/* Create the stuff inside our window. */
HWND text;
void init_main_window(HWND parent)
{
	HWND edit;
	HFONT font;

	/* Create help text */
	text = CreateWindowEx(0, "STATIC", instructions,
			WS_CHILD | WS_VISIBLE | SS_CENTER | SS_SUNKEN,
			0, 0,
			400, 20,
			parent,
			NULL,
			GetModuleHandle(NULL),
			NULL);
	font = GetStockObject(DEFAULT_GUI_FONT);
	SendMessage(text, WM_SETFONT, (WPARAM)font, MAKELPARAM(FALSE, 0));
	/* Create edit control */
	edit = CreateWindowEx(WS_EX_CLIENTEDGE, "EDIT", "",
			WS_CHILD | WS_VISIBLE | WS_VSCROLL | WS_HSCROLL |
			ES_MULTILINE | ES_AUTOVSCROLL | ES_AUTOHSCROLL,
			0, 20,
			400, 200,
			parent,
			(HMENU)IDC_MAIN_EDIT,
			GetModuleHandle(NULL),
			NULL);
	SendMessage(edit, WM_SETFONT, (WPARAM)font, MAKELPARAM(FALSE, 0));
	SetDlgItemText(parent, IDC_MAIN_EDIT, blocked_sites);
}

/*********************/
/*   EVENT HANDLER   */
/*********************/
int _hf_defaults;
LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	HWND edit;
	RECT rect;

	switch(msg) {
		case WM_CREATE:
			init_main_window(hwnd);
			break;
		case WM_CLOSE:
			edit = GetDlgItem(hwnd, IDC_MAIN_EDIT);
			if(_hf_defaults == 1 ||
			   SendMessage(edit, EM_GETMODIFY, 0, 0) == TRUE) {
				int r = MessageBox(NULL,
						"Save changes before exit?",
						"Save Changes",
						MB_ICONQUESTION | MB_YESNO);
				if(r == IDYES) {
					int s;
					if(save(hwnd) == 0) {
						s = MessageBox(NULL,
						  "Changes could not be saved!"\
						  "  Quit anyway?",
						  "Error",
						  MB_ICONERROR | MB_YESNO);
						if(s == IDNO)
							break;
					}
				}
			}
			DestroyWindow(hwnd);
			break;
		case WM_DESTROY:
			free(blocked_sites);
			PostQuitMessage(0);
			break;
		case WM_SIZE:
			{
			SIZE text_size;
			int height;
			GetClientRect(hwnd, &rect);
			/* Use the correct height of the text */
			GetTextExtentPoint32(GetDC(text), instructions,
					strlen(instructions), &text_size);
			height = text_size.cy;
			SetWindowPos(text, NULL, 0, 0, rect.right, height,
					SWP_NOZORDER);
			edit = GetDlgItem(hwnd, IDC_MAIN_EDIT);
			SetWindowPos(edit, NULL, 0, height, rect.right,
					rect.bottom - height,
					SWP_NOZORDER);
			break;
			}
		case WM_COMMAND:
			switch(LOWORD(wParam)) {
				case ID_FILE_EXIT:
					PostMessage(hwnd, WM_CLOSE, 0, 0);
					break;
				case ID_FILE_SAVE:
					save(hwnd);
					break;
				case ID_EDIT_CUT:
					SendDlgItemMessage(hwnd, IDC_MAIN_EDIT,
							WM_CUT, 0, 0);
					break;
				case ID_EDIT_COPY:
					SendDlgItemMessage(hwnd, IDC_MAIN_EDIT,
							WM_COPY, 0, 0);
					break;
				case ID_EDIT_PASTE:
					SendDlgItemMessage(hwnd, IDC_MAIN_EDIT,
							WM_PASTE, 0, 0);
					break;
				case ID_HELP_ABOUT:
					create_about_window(hwnd);
					break;
			}
			break;
		default:
			return DefWindowProc(hwnd, msg, wParam, lParam);
	}
	return 0;
}

/*******************/
/*   ENTRY POINT   */
/*******************/

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
		LPSTR lpCmdLine, int nCmdShow)
{
	WNDCLASSEX wc;
	HWND hwnd;
	MSG msg;

	/* Register the main window class */
	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc = WndProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON));
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
	wc.lpszMenuName = MAKEINTRESOURCE(IDR_MAINMENU);
	wc.lpszClassName = winnocence_class;
	wc.hIconSm = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON));
	RegisterClassEx(&wc);
	/* Register the about window class */
	ac.cbSize = sizeof(WNDCLASSEX);
	ac.style = 0;
	ac.lpfnWndProc = AboutProc;
	ac.cbClsExtra = 0;
	ac.cbWndExtra = 0;
	ac.hInstance = hInstance;
	ac.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	ac.hCursor = LoadCursor(NULL, IDC_ARROW);
	ac.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
	ac.lpszMenuName = NULL;
	ac.lpszClassName = about_class;
	ac.hIconSm = LoadIcon(NULL, IDI_APPLICATION);
	RegisterClassEx(&ac);

	/* Load our lines from the hosts file */
	blocked_sites = get_winnocence_lines();
	if(blocked_sites == NULL) {
		static char errmsg_pre[] = "Could not open hosts file!  " \
			"Make sure you have permission to open ";
		char *badpath = get_hostsfile();
		char *errmsg = malloc(strlen(errmsg_pre)+strlen(badpath)+1);
		strcpy(errmsg, errmsg_pre);
		strcat(errmsg, badpath);
		MessageBox(NULL, errmsg, "Error!", MB_ICONERROR | MB_OK);
		free(badpath);
		free(errmsg);
		return 1;
	}

	/* Create our window */
	hwnd = CreateWindowEx(
			0,
			winnocence_class,
			title_version,
			WS_OVERLAPPEDWINDOW,
			CW_USEDEFAULT, CW_USEDEFAULT,
			400, 200,
			NULL,
			NULL,
			hInstance,
			NULL);
	ShowWindow(hwnd, nCmdShow);
	UpdateWindow(hwnd);

	/* Event loop */
	while(GetMessage(&msg, NULL, 0, 0) > 0) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return msg.wParam;
}
